/*
 * JZ4780 HDMI peripheral support.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/hw_mmio_register_block.h>

// HDMI device control.

class Hdmi_jz4780_chip
{
private:
  l4_addr_t _start, _end;
  l4_cap_idx_t _irq;
  Hw::Register_block<8> _regs;

  // Identification.

  uint16_t _version;

  // Transfer properties.

  bool _segment_read;
  uint8_t _device_register;

protected:
  void get_identification();

  void irq_init();

  void i2c_init();
  long i2c_wait();

  void hotplug_init();

public:
  Hdmi_jz4780_chip(l4_addr_t start, l4_addr_t end, l4_cap_idx_t irq);

  void get_version(uint8_t *major, uint16_t *minor);

  int i2c_read(uint8_t *buf, unsigned int length);
  void i2c_set_address(uint8_t address);
  void i2c_set_segment(uint8_t segment);
  void i2c_set_register(uint8_t device_register);

  bool connected();
  long wait_for_connection();
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *jz4780_hdmi_init(l4_addr_t start, l4_addr_t end, l4_cap_idx_t irq);

void jz4780_hdmi_get_version(void *hdmi, uint8_t *major, uint16_t *minor);

int jz4780_hdmi_i2c_read(void *hdmi, uint8_t *buf, unsigned int length);

void jz4780_hdmi_i2c_set_address(void *hdmi, uint8_t address);

void jz4780_hdmi_i2c_set_segment(void *hdmi, uint8_t segment);

void jz4780_hdmi_i2c_set_register(void *hdmi, uint8_t device_register);

int jz4780_hdmi_connected(void *hdmi);

long jz4780_hdmi_wait_for_connection(void *hdmi);

EXTERN_C_END
